package rabinizer.exec;

import java.io.File;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;

import rabinizer.formulas.Formula;
import rabinizer.parser.LTLParser;

public class Prism {

	public static KDFA DGRA;

	public static GeneralizedRabinAcceptanceCondition GRAC;	
	public static GenAccCondPrism conditions = null;




	/*
	 * degeneralized DRA for PRISM
	 */
	public static String createRabinizerDRA(String formula, ArrayList<String> ap) {
		computeDGRAandGRAC(formula);
		long start_time = System.currentTimeMillis();
    	// Phase 6: Construction of the Rabin Automaton
    	Misc.verboseln("Phase 6: Rabin Automaton");
    	KRabinAutomaton rabin = new KRabinAutomaton(DGRA, GRAC);
        System.out.println("\n\nRabin Transition Automaton: ");
        System.out.println("\tNumber of states    : " + rabin.states.size());
        System.out.println("\tNumber of edges     : " + rabin.edges.size());

    	// Phase 7: Construction of Rabin Acceptance Condition
    	Misc.verboseln("Phase 7: Construction of Rabin Acceptance Condition");
    	KRabinAcceptanceCondition rabinAC = new KRabinAcceptanceCondition(rabin);
        System.out.println("\tSize of Rabin AC    : " + rabinAC.size());
        
     // Phase 8: Construction of the state-based Rabin Automaton
    	Misc.verboseln("Phase 8: state-based Rabin Automaton");
    	KRabinAutomatonStates rabinS = new KRabinAutomatonStates(rabin, rabinAC);
        System.out.println("\n\nRabin State Automaton: ");
        System.out.println("\tNumber of states    : " + rabinS.states.size());
        System.out.println("\tNumber of edges     : " + rabinS.edges.size());

    	// Phase 9: Construction of the state-based Rabin Acceptance Condition
    	Misc.verboseln("Phase 9: Construction of the state-based Rabin Acceptance Condition");
    	KRabinStatesAcceptanceCondition rabinSAC = new KRabinStatesAcceptanceCondition(rabinS);
        System.out.println("\tSize of Rabin AC    : " + rabinAC.size());
    	long rabinstate_time = System.currentTimeMillis();
    	System.out.println("\tTime to construct DRA  : " + (rabinstate_time - start_time) + "msec");
        //System.out.println("$$$$$$$$$$$$$$$$$$   : " + rabinS.statesToNumbers);    	
    	String result = rabinS.toPrism(ap.size());
    	result += "\n" + rabinAC.size() + " acceptance pairs: \n";
        result += rabinSAC.toPrism(rabinS.statesToNumbers);
        //System.out.println(result);
        conditions = null;
		return result;
	}


	/*
	 * transition-based generalized DRA for PRISM
	 */
	public static String createRabinizerTransitionGeneralizedDRA(String formula, ArrayList<String> ap) {
		computeDGRAandGRAC(formula);
		conditions = null;
		return DGRA.toPrism(ap.size());
	}
	
	/*
	 * state-based generalized DRA for PRISM
	 */
	public static String createRabinizerStateGeneralizedDRA(String formula, ArrayList<String> ap) {
		computeDGRAandGRAC(formula);
		// Phase 10: Construction of the state-based Generalized Rabin Automaton
    	Misc.verboseln("\nPhase 10: state-based Generalized Rabin Automaton");
    	long start_time = System.currentTimeMillis();
    	KDFAStates kdfaS = new KDFAStates(DGRA, GRAC);
        System.out.println("\n\n State-based generalized DRA : ");
        System.out.println("\tNumber of states    : " + kdfaS.states.size());
        System.out.println("\tNumber of edges     : " + kdfaS.edges.size());
    	//Phase 11: Construction of the state-based Generalized Rabin Acceptance Condition
    	Misc.verboseln("");
    	Misc.verboseln("Phase 11: Construction of the state-based Generalized Rabin Acceptance Condition");
    	GeneralizedRabinStatesAcceptanceCondition grabinSAC = new GeneralizedRabinStatesAcceptanceCondition(kdfaS);
    	long DGRAS_time = System.currentTimeMillis();
        System.out.println("\tSize of Rabin AC    : " + grabinSAC.size());
        System.out.println("\tTime for state-based GRA : " + (DGRAS_time - start_time) + "msec");
        
        String result = kdfaS.toPrism(ap.size());
        //result += "\n" + grabinSAC.size() + " acceptance pairs: \n";
        
        conditions = grabinSAC.toBitSet(kdfaS.statesToNumbers);
		return result;
	}
		
		
		
	public static void computeDGRAandGRAC(String prismFormula) {	
		Misc.verbose = false;
		// Phase 1: Parsing of the input formula
		Misc.verboseln("\nPhase 1: Parsing of the input formula\n");
		long start_time = System.currentTimeMillis();
		// remove "" and L added by PRISM
		Misc.verboseln(prismFormula);
		String formula = prismFormula.replace('\"',' ').replace('L',' ');
		Misc.verboseln(formula);
		StringReader reader = new StringReader(formula);
		LTLParser parser = null;
		try { parser = new LTLParser(reader); }
		catch (Exception e) {
			errorMessageAndExit("Exception when reading the input formula: " + e.getLocalizedMessage()); }
		Formula phi = null;
		try { phi = parser.parse(); }
		catch (Exception e) {
			errorMessageAndExit("Exception when parsing: " + e.getLocalizedMessage()); }
		System.out.println("Input formula: " + phi.toString());
		phi = phi.toNNF();
		System.out.println("Input formula in negation normal form: " + phi);

		// Phase 2: Enumeration of valuations
		Misc.verboseln("\nPhase 2: Enumeration of valuations");
		Misc.initializeValuations(Misc.bijectionIdAtom.size());
		BDDService.init(Misc.bijectionIdAtom.size());

		// Phase 3: Construction of segment automata
		Misc.verboseln("\n\nPhase 3: Construction of segment automata");

		// Phase 3a: Construction of always- and eventually- recurrent formulas aRF1, ..., aRFm, eRF1, ..., eRFn
		List<Formula> alwaysRecFs = new ArrayList<Formula>();
		alwaysRecFs.addAll(phi.argumentsAlways()); //(recurrentFormulas.left);
		List<SegmentAutomaton> alwaysSAs = new ArrayList<SegmentAutomaton>(alwaysRecFs.size());
		List<Set<Formula>> alwaysISSs = new ArrayList<Set<Formula>>(alwaysRecFs.size());
		for (Formula aRF : alwaysRecFs) {
			// segment automaton
			//Misc.verboseln("\nAlways segment automaton");
			SegmentAutomaton aSA = new SegmentAutomaton(aRF, Misc.valuations);
			alwaysSAs.add(aSA);
			// corresponding initial segment state
			Set<Formula> iSS = new HashSet<Formula>();
			if (aSA.finalStates.contains(Misc.fTrue)) {
				iSS.add(Misc.fTrue);
			} else {
				boolean pickedFS = false;
				for(Formula f : aSA.finalStates) {
					if (!f.equals(Misc.fFalse)) {
						iSS.add(f);
						pickedFS = true;
						break;
					}
				}
				if (!pickedFS) iSS.add(Misc.fFalse);
			}
			Misc.verboseln("(always) Final state for iSS: "+iSS);
			iSS = aSA.backwardsReachability(iSS);
			Misc.verboseln("(always) Initial segment state: "+iSS);
			alwaysISSs.add(iSS);
		}

		// Phase 4: Construction of DFA A
		Misc.verboseln("\n\nPhase 4: Construction of Generalized Rabin Automaton");
		List<SegmentAutomaton> segmentAutomata = alwaysSAs;
		Misc.verboseln("");
		KDFA dfaA = new KDFA(phi, segmentAutomata, new KState(alwaysSAs).initialize(), Misc.valuations);
		KDFA dfaA2 = new KDFA(phi, segmentAutomata, new KState(alwaysSAs).initializeFilled(), Misc.valuations);
		if(dfaA2.states.size() < dfaA.states.size()){
			dfaA = dfaA2;
			System.out.println("\n\nInitial states were optimized");
		}else
			System.out.println("\n\nStandard initial states were used");
		long gra_time = System.currentTimeMillis();

		System.out.println("\nGeneralized Rabin Automaton: ");
		System.out.println("\tNumber of states             : " + dfaA.states.size());
		System.out.println("\tNumber of edges              : " + dfaA.edges.size());
		System.out.println("\tTime to construct DGRA       : " + (gra_time - start_time) + "msec");
		DGRA = dfaA;

		// Phase 5: Construction of Generalized Rabin Acceptance Condition
		Misc.verboseln("\n\nPhase 5: Construction of Generalized Rabin Acceptance Condition");
		Misc.verboseln("");
		List<Formula> subformulasAlways = new ArrayList<Formula>();
		subformulasAlways.addAll(phi.argumentsAlways());
		Misc.verboseln("G-subformulas: "+subformulasAlways);
		// set numbered choice
		GeneralizedRabinAcceptanceCondition genRabinAC = new GeneralizedRabinAcceptanceCondition(subformulasAlways, dfaA, 2); //format irrelevant(?)
		long grac_time = System.currentTimeMillis();
		System.out.println("\tNumber of pairs              : " + genRabinAC.size());
		System.out.println("\tTime to construct condition  : " + (grac_time - gra_time) + "msec");
		GRAC = genRabinAC;
		
		return;
	}

	/*
	public static String getTranGRAnoGRAC(int NrOfAp) {
		String a = "" + DGRA.states.size() + " states "
				   + "(start " + (DGRA.statesToNumbers.get(DGRA.initialState)-1) + "), " 
				   + NrOfAp + " labels:\n";
		for (Entry<Tuple<Tuple<Formula, KState>, Tuple<Formula, KState>>, ValuationSet> edge : DGRA.edges.entrySet()) {
			for (Valuation v : edge.getValue()) {
				a += " " + (DGRA.statesToNumbers.get(edge.getKey().left)-1) + "-";
				a += v.toString();
				a += "->" + (DGRA.statesToNumbers.get(edge.getKey().right)-1) + "\n";
			}
		}
		conditions = null;
		return a + "; ";
	}
	*/
	
	/*

	public String toStringPrism(APTable table, GenAccCondList conds,
			GenAccCondPrism prismConds) {

		StringBuilder result = new StringBuilder();

		getInitialState().setId(0);
		int counter = 1;
		for (AutomatonState s : allStates) {
			if (s != getInitialState())
				s.setId(counter++);
		}

		result.append(allStates.size() + " states (start 0), ");
		result.append(table.getNumberOfAPs() + " labels: ");
		// Append edges
		Valuation v = new Valuation(table);
		for (AutomatonState s : allStates) {
			for (AutomatonEdge edge : s.getSuccs()) {
				stateIterVals(result, 0, v, edge);
			}
		}
		// Append acceptance conditions
		result.append("; " + conds.getConditions().size()
				+ " GENERALIZED RABIN CONDITIONS. ");

		for (int i = 0; i < conds.getConditions().size(); i++) {

			BitSet fin = new BitSet();
			ArrayList<BitSet> infs = new ArrayList<BitSet>();

			GenAccCond cond = conds.getConditions().get(i);
			// finite state set
			for (AutomatonState s : getAllStates()) {
				if (cond.getFinStates().contains(s)) {
					fin.set(s.getId());
				}
			}
			System.out.println("***NEW FIN SET:" + fin.toString());

			// infinite state sets
			for (StateSet set : cond.getInfStates()) {

				BitSet singleSet = new BitSet();
				for (AutomatonState s : getAllStates()) {
					if (set.contains(s)) {
						singleSet.set(s.getId());
					}
				}
				infs.add(singleSet);
				System.out.println("***NEW INF SET:" + singleSet.toString());
			}

			// add the new condition
			prismConds.addAccCond(fin, infs);

		}
		return result.toString();

	}

	 */
	
	
	public static void errorMessageAndExit(String text) {
		System.out.println("ERROR: " + text);
		System.exit(1);
	}	
	
	public static void tryout(){
		ArrayList<String> l = new ArrayList<String>();
		l.add("0"); 
		l.add("1");
		System.out.println(createRabinizerTransitionGeneralizedDRA("G F \"L0\" & G F \"L1\"",l));
	}

}
