package rabinizer.exec;

import java.util.*;
import rabinizer.formulas.*;
import net.sf.javabdd.*;

/**
 * A valuation is a map from atomic propositions to boolean values.
 * In the AST atomic propositions are integers.
 * 
 * @author Ruslan Ledesma-Garza
 *
 */
public class Valuation extends HashMap<Integer, Boolean> {
	
	/**
	 * An automatically generated serial version.
	 */
	private static final long serialVersionUID = -5927454815102468383L;
	
	private static BDDFactory bf;

	public Valuation() {
		super();
	}
	
	public Valuation(Valuation v) {
		for(Integer i:v.keySet()){
			this.put(i, v.get(i));
		}
	}
	
	public Valuation(boolean[] values) {
		super();
		for (int i = 0; i < values.length; i++){
			this.put(i, values[i]);
		}
	}
	
	public Valuation(List<Boolean> values) {
		super();
		for (int i = 0; i < values.size(); i++){
			this.put(i, values.get(i));
		}
	}
	
	private String strValuation = null;
	public String toString() {
		if (strValuation == null) {
			strValuation = "{";
			boolean first = true;
			for(Map.Entry<Integer, Boolean> e :this.entrySet()) {
				if (first && e.getValue().booleanValue()) {
					String v = Misc.bijectionIdAtom.atom(e.getKey().intValue()); 
					if (v == null) {
						strValuation = strValuation + "v" + e.getKey().intValue();
					} else {
						strValuation = strValuation + v;
					}
					first = false;
				} else {
					String v = Misc.bijectionIdAtom.atom(e.getKey().intValue()); 
					if (v == null) {
						strValuation = strValuation + (e.getValue().booleanValue() ? ", v" + e.getKey().intValue() : "");
					} else {
						strValuation = strValuation + (e.getValue().booleanValue() ? ", " + v : "");;
					}
					
				}			
			}
			strValuation = strValuation + "}";
		}
		return strValuation;
	}
	
	public Formula toFormula() {
		Formula result = null;
		for(Map.Entry<Integer, Boolean> e :this.entrySet()) {
			Literal l = new Literal(Misc.bijectionIdAtom.atom(e.getKey()), e.getKey(), !e.getValue());
			if (result == null) {
				result = l;
			} else {
				result = new Conjunction(result, l);
			}
		}
		return result;
	}
	
/*	
	public BDD toBDD() {
		BDD result = bf.one();  // BDD for True
		for(Map.Entry<Integer, Boolean> e :this.entrySet()) {
			Literal l = new Literal(Misc.bijectionIdAtom.atom(e.getKey()), e.getKey(), !e.getValue());
			result = result.and(l.bdd());
		}
		return result;
	}
*/

//	@Override
//	public int compareTo(Object o) {
//		Valuation that = (Valuation)o;
//		if (this.size() != that.size())
//			return this.size() - that.size();
//		else {
//			List<Boolean> thisvals = new ArrayList<Boolean>(this.size());
//			for (Map.Entry<Integer, Boolean> e : this.entrySet())
//				thisvals.add(e.getValue());
//			List<Boolean> thatvals = new ArrayList<Boolean>(that.size());
//			for (Map.Entry<Integer, Boolean> e : that.entrySet())
//				thatvals.add(e.getValue());
//			for (int i=0; i < thisvals.size(); i++) {
//				if (thisvals.get(i).compareTo(thatvals.get(i))) 
//			}
//		}
//	}
}
