package rabinizer.formulas;

import java.util.*;

import net.sf.javabdd.*;
import rabinizer.exec.*;

/**
 * @author Andreas Gaiser & Ruslan Ledesma-Garza
 *
 */
public class FOperator extends Formula {

	public Formula operand;
	
	public FOperator(Formula f) {
		this.operand = f;
	}

	public BDD bdd() {
		if (cachedBdd == null) {
			Formula booleanAtom = new FOperator(operand.representative());
			int bddVar = Misc.bijectionBooleanAtomBddVar.id(booleanAtom);
			if (Misc.bddFactory.varNum() <= bddVar) Misc.bddFactory.extVarNum(bddVar);
			cachedBdd = Misc.bddFactory.ithVar(bddVar);
			Misc.representativeOfBdd(cachedBdd, this);
		}
		return cachedBdd;
	}
	
	@Override
	public int hashCode() {
		return operand.hashCode()*4;
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof FOperator)) return false;
		else { 
			return ((FOperator)o).operand.equals(this.operand);
		}
	}
	
	@Override
	public Formula negated() {
		return new GOperator(operand.negated());
	}

           	@Override
        public String toReversePolishString() {
                    return "F " + operand.toReversePolishString();
        }
	
	@Override
	public String toString() {
		if (strFormula == null) strFormula = "F " + operand.toString();
		return strFormula;
	}
	
	@Override
	public Tuple<Set<Formula>, Set<Formula>> recurrentFormulas(boolean accumulate) {
            Tuple<Set<Formula>, Set<Formula>> rfs = null;
                rfs = operand.recurrentFormulas(accumulate);
		if (accumulate && operand.isProgressFormula()) rfs.right.add(operand);
		return rfs;
	}

    @Override
	public Formula evaluateCurrentAssertions(Valuation valuation) {
            return this;
	}
    
	@Override
	public Formula removeConstantCurrentAssertions() {
		return this;
	}
	
	@Override
	public Formula removeX() {
		return this;
	}	

	@Override
	public Formula unfold() {
		// U(F phi) = U(phi) \/ X F U(phi)
		return new Disjunction(operand.unfold(), new XOperator(new FOperator(operand)));
	}

	@Override
	public Formula unfoldNoG() {
		// U(F phi) = U(phi) \/ X F U(phi)
		return new Disjunction(operand.unfoldNoG(), new XOperator(new FOperator(operand)));
	}
	
	@Override
	public boolean isProgressFormula() {
		return operand.isProgressFormula();
	}
	
	@Override
	public boolean containsG() {
		return operand.containsG();
	}

    @Override
	public boolean hasSubformula(Formula f) {
        return this.equals(f) || operand.hasSubformula(f);
	}

        @Override
        public Formula removeXsFromCurrentBooleanAtoms() {
            return this;
        }

    @Override
	public Set<Formula> argumentsAlways() {
            return operand.argumentsAlways();
	}


               @Override
            public Set<Formula> argumentsFinsideG(boolean acc) {
                   Set<Formula> r = operand.argumentsFinsideG(acc);
                   if (acc) r.add(operand);
                   return r;
	}
               
    public boolean untilOcurrs() {
	   return operand.untilOcurrs();
  	}
    
    public Formula toNNF() {
    	return new FOperator(operand.toNNF());
    }
}
