package rabinizer.formulas;

import java.util.*;

import rabinizer.exec.*;
import net.sf.javabdd.*;

public class Literal extends Formula {

	public String atom;
	public int atomId;
	public boolean negated;

	public Literal(String atom, int atomId, boolean negated) {
		this.atom = atom;
		this.atomId = atomId;
		this.negated = negated;
	}
	
	public Literal(Literal l) {
		this.atom = l.atom;
		this.atomId = l.atomId;
		this.negated = l.negated;
	}
	
	public BDD bdd() {
		if (cachedBdd == null) {
			int bddVar = Misc.bijectionBooleanAtomBddVar.id(this);
			if (Misc.bddFactory.varNum() <= bddVar)	Misc.bddFactory.extVarNum(1);
			cachedBdd = (negated ? Misc.bddFactory.ithVar(bddVar) : Misc.bddFactory.nithVar(bddVar));
			Misc.representativeOfBdd(cachedBdd, this);
		}
		return cachedBdd;
	}

	@Override
	public int hashCode() {
		return atomId;
	}

	public boolean equals(Object o) {
		if (!(o instanceof Literal))
			return false;
		else {
			return ((Literal) o).atomId == this.atomId && ((Literal)o).negated == this.negated;
		}
	}
	
	@Override
	public Formula negated() {
		return new Literal(atom, atomId, !negated);
	}

	@Override
	public String toReversePolishString() {
		return strFormula = (negated ? "! " : "") + atom;
	}
	
	@Override
	public String toString() {
		if (strFormula == null) strFormula = (negated ? "!" : "") + atom;
		return strFormula;
	}
	
	@Override
	public Tuple<Set<Formula>, Set<Formula>> recurrentFormulas(boolean accumulate) {
            return new Tuple<Set<Formula>, Set<Formula>>(new HashSet<Formula>(), new HashSet<Formula>());
	}

	@Override
	public Formula evaluateCurrentAssertions(Valuation valuation) {
		return new BooleanConstant(valuation.get(atomId).booleanValue() ^ negated);
	}

	@Override
	public Formula removeConstantCurrentAssertions() {
		return this;
	}
	
	@Override
	public Formula removeX() {
		return this;
	}

	@Override
	public Formula unfold() {
		return this;
	}

	@Override
	public Formula unfoldNoG() {
		return this;
	}
	
	@Override
	public boolean isProgressFormula() {
		return true;
	}
	
	@Override
	public boolean isVeryDifferentFrom(Formula f) {
		return ! f.hasSubformula(this);
	}

        @Override
	public boolean hasSubformula(Formula f) {
            return this.equals(f);
	}

        @Override
        public Formula removeXsFromCurrentBooleanAtoms() {
            return this;
        }

        @Override
	public Set<Formula> argumentsAlways() {
            return new HashSet<Formula>();
	}

               @Override
            public Set<Formula> argumentsFinsideG(boolean acc) {
            return new HashSet<Formula>();
	}


    public boolean untilOcurrs() {
    	return false;
    }
    
    public Formula toNNF() {
    	return this;
    }
}
