package tests;

import static org.junit.Assert.*;

import java.io.StringReader;
import java.util.*;

import org.junit.Test;

import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import rabinizer.formulas.Formula;
import rabinizer.parser.LTLParser;

@RunWith(value = Parameterized.class)
public class Representative {

	private String canonical;
	private String member;	
	
	public Representative(String formula1, String formula2) {
		this.canonical = formula1;
		this.member = formula2;
	}

	@Parameters
	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] {
				{ "a", "(a & b) | (a & !b)" }, // TODO: pass this test
				{ "a", "a & (b | !b)" },       // TODO: pass this test
				
				{ "a", "a" },
				{ "X a", "X a" },
				{ "F a", "F a" },
				{ "G a", "G a" },
				{ "a U b", "a U b" },

				{ "a", "a | a" },				
				{ "a", "a & a" },
				{ "a", "true & a" },
				{ "a", "a & true" },
				
				{ "a & b", "a & b" },
				{ "a & b", "b & a" },
				{ "a | b", "a | b" },
				{ "a | b", "b | a" },
								
				{ "a & b", "(a | a) & b" },
				{ "a & b", "(a & a) & b" },
				{ "a & b", "a & (b & a)" },  // TODO: pass this test
				
				{ "a", "(a | b) & (a | !b)" },

				{ "true", "true | a" },
				{ "true", "a | true" },
				{ "false", "false & a" },
				{ "false", "a & false" },
				{ "false", "(a & false) & b" },
				{ "true", "(a | true) & true" },

				{ "X a", "true & X a" },
				{ "a", "false | a" },
				
				{ "X a", "X a & X a" },
				{ "X a", "(X a & X a) & X a" },
				{ "X a", "(X a & X a) & (X a & X a)" },
				{ "X a", "(X a & X a) | (X a & X a)" },
				{ "a & X a", "(a & X a) | (a & X a)" },
				{ "a & F X a", "(a & F X a) | (a & F X a)" },
				{ "a | X a", "(a | X a) & (a | X a)" },
				{ "a | X F a", "(a | X F a) & (a | X F a)" },
				{ "(a | X F a) & b", "(a | X F a) & b & (a | X F a)" },

				{ "a & b", "b & a & b" },
				
				{ "a & b", "a & a & b" },
				
				{ "(a | F a)", "(a | F a) & (a | F a) & b" },
			
				{ "(a | X F a) & b", "(a | X F a) & (a | X F a) & b" },
								
				{ "G (a & F b)", "(false | (true & G (a & F b)))" },
				
				{ "(a | X F a) & X G F a", "(a | X F a) & (a | X F a) & X G F a" },
		};
		return Arrays.asList(data);
	}

	@Test
	public void pushTest() {
		System.out.println("\n"+this.getClass().getCanonicalName());
		
		LTLParser parser = new LTLParser(new StringReader(canonical));
		Formula cf = null;
		try { cf = parser.parse(); }
		catch (Exception e) { assertTrue(false); }
		
		parser.ReInit(new StringReader(member));
		Formula mf = null;
		try { mf = parser.parsePreviousAtoms(); }
		catch (Exception e) { assertTrue(false); }
		
		cf.representative();
		mf.representative();
		
		System.out.println("member  : " + mf);
		System.out.println("actual  : " + mf.representative());
		System.out.println("expected: " + cf);
		System.out.println("exp.repr: " + cf.representative());
		assertTrue(mf.representative().equals(cf.representative()));
	}
}
