package tests;

import static org.junit.Assert.*;


import java.io.StringReader;
import java.util.Arrays;
import java.util.Collection;

import org.junit.Test;

import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import rabinizer.formulas.Formula;
import rabinizer.parser.LTLParser;

@RunWith(value = Parameterized.class)
public class ReversePolishNotation {

	private String input;
	private String output;
	private boolean expected;	
	
	public ReversePolishNotation(String input, String output, boolean expected) {
		this.input = input;
		this.output = output;
		this.expected = expected;
	}

	@Parameters
	public static Collection<Object[]> data() {
		Object[][] data = new Object[][] { 
				{ "X (a | F b)", "X | a F b", true },
				{ "a & ((G F a | F G b) & (G F c | F G d))  | ! a & ((G F e | F G f) & (G F g | F G h))",
				  "| & a & | G F a F G b | G F c F G d & ! a & | G F e F G f | G F g F G h", true }
				
		};
		return Arrays.asList(data);
	}

	@Test
	public void pushTest() {
		System.out.println("\n"+this.getClass().getCanonicalName());
		
		LTLParser parser = new LTLParser(new StringReader(input));
		Formula inf = null;
		try { inf = parser.parse(); }
		catch (Exception e) { assertTrue(false); }
		
		System.out.println("input : "+inf);
		System.out.println("actual: "+inf.toReversePolishString());
		System.out.println("output: "+output);
		System.out.println("expected: "+expected);
		assertTrue(inf.toReversePolishString().equals(output) == expected);
	}
}
