package tests;

import static org.junit.Assert.*;

import java.io.StringReader;
import java.util.*;

import org.junit.Test;

import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import rabinizer.exec.Valuation;
import rabinizer.formulas.Formula;
import rabinizer.parser.LTLParser;

@RunWith(value = Parameterized.class)
public class TemporalStep {

	private String input;
	private Valuation valuation;
	private String expected;	
	
	public TemporalStep(String input, String expected) {
		this.input = input;
		boolean[] values = {true, false};
		this.valuation = new Valuation(values);
		this.expected = expected;
	}

    @Parameters
	public static Collection<Object[]> data() {
        Object[][] data = new Object[][] { 
            { "(a & X b) | (b & X a)", "b" },
            { "true & X a & G b", "a & G b" },
            { "X F a | F a", "F a | F a" },
            { "a U b", "a U b" },
            { "! two", "false" },
        };
        return Arrays.asList(data);
    }

	@Test
	public void pushTest() {
		System.out.println(this.getClass().getCanonicalName()+" : "+input+" and "+expected);
		
		LTLParser parser = new LTLParser(new StringReader(input));
		Formula inf = null;
		try { inf = parser.parse(); }
		catch (Exception e) { assertTrue(false); }
		
		parser.ReInit(new StringReader(expected));
		Formula exf = null;
		try { exf = parser.parsePreviousAtoms(); }
		catch (Exception e) { assertTrue(false); }
		
		assertTrue(exf.equals(inf.temporalStep(valuation)));
	}
}
